package cc.toph.simplycompat.registry;

import cc.toph.simplycompat.SimplyCompat;
import cc.toph.simplycompat.item.SimplyCompatToolMaterials;
import dev.architectury.registry.registries.DeferredRegister;
import dev.architectury.registry.registries.RegistrySupplier;
import net.minecraft.core.registries.Registries;
import net.minecraft.world.item.Item;
import net.sweenus.simplyswords.config.ConfigDefaultValues;
import net.sweenus.simplyswords.item.SimplySwordsSwordItem;

public class ItemsRegistry {

  enum WeaponType {
    CHAKRAM(
      "chakram",
      net.sweenus.simplyswords.config.Config.getFloat(
        "chakram_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.chakram_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "chakram_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.chakram_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "chakram_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.chakram_attackSpeed
      )
    ),
    CLAYMORE(
      "claymore",
      net.sweenus.simplyswords.config.Config.getFloat(
        "claymore_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.claymore_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "claymore_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.claymore_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "claymore_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.claymore_attackSpeed
      )
    ),
    CUTLASS(
      "cutlass",
      net.sweenus.simplyswords.config.Config.getFloat(
        "cutlass_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.cutlass_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "cutlass_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.cutlass_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "cutlass_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.cutlass_attackSpeed
      )
    ),
    GLAIVE(
      "glaive",
      net.sweenus.simplyswords.config.Config.getFloat(
        "glaive_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.glaive_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "glaive_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.glaive_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "glaive_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.glaive_attackSpeed
      )
    ),
    GREATAXE(
      "greataxe",
      net.sweenus.simplyswords.config.Config.getFloat(
        "greataxe_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.greataxe_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "greataxe_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.greataxe_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "greataxe_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.greataxe_attackSpeed
      )
    ),
    GREATHAMMER(
      "greathammer",
      net.sweenus.simplyswords.config.Config.getFloat(
        "greathammer_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.greathammer_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "greathammer_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.greathammer_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "greathammer_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.greathammer_attackSpeed
      )
    ),
    HALBERD(
      "halberd",
      net.sweenus.simplyswords.config.Config.getFloat(
        "halberd_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.halberd_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "halberd_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.halberd_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "halberd_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.halberd_attackSpeed
      )
    ),
    KATANA(
      "katana",
      net.sweenus.simplyswords.config.Config.getFloat(
        "katana_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.katana_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "katana_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.katana_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "katana_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.katana_attackSpeed
      )
    ),
    LONGSWORD(
      "longsword",
      net.sweenus.simplyswords.config.Config.getFloat(
        "longsword_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.longsword_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "longsword_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.longsword_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "longsword_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.longsword_attackSpeed
      )
    ),
    RAPIER(
      "rapier",
      net.sweenus.simplyswords.config.Config.getFloat(
        "rapier_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.rapier_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "rapier_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.rapier_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "rapier_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.rapier_attackSpeed
      )
    ),
    SAI(
      "sai",
      net.sweenus.simplyswords.config.Config.getFloat(
        "sai_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.sai_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "sai_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.sai_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "sai_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.sai_attackSpeed
      )
    ),
    SCYTHE(
      "scythe",
      net.sweenus.simplyswords.config.Config.getFloat(
        "scythe_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.scythe_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "scythe_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.scythe_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "scythe_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.scythe_attackSpeed
      )
    ),
    SPEAR(
      "spear",
      net.sweenus.simplyswords.config.Config.getFloat(
        "spear_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.spear_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "spear_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.spear_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "spear_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.spear_attackSpeed
      )
    ),
    TWINBLADE(
      "twinblade",
      net.sweenus.simplyswords.config.Config.getFloat(
        "twinblade_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.twinblade_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "twinblade_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.twinblade_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "twinblade_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.twinblade_attackSpeed
      )
    ),
    WARGLAIVE(
      "warglaive",
      net.sweenus.simplyswords.config.Config.getFloat(
        "warglaive_positiveDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.warglaive_positiveDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "warglaive_negativeDamageModifier",
        "WeaponAttributes",
        ConfigDefaultValues.warglaive_negativeDamageModifier
      ),
      net.sweenus.simplyswords.config.Config.getFloat(
        "warglaive_attackSpeed",
        "WeaponAttributes",
        ConfigDefaultValues.warglaive_attackSpeed
      )
    );

    final String weaponNameSuffix;
    final float positiveModifier;
    final float negativeModifier;
    final float attackSpeed;

    WeaponType(
      String weaponNameSuffix,
      float positiveModifier,
      float negativeModifier,
      float attackSpeed
    ) {
      this.weaponNameSuffix = weaponNameSuffix;
      this.positiveModifier = positiveModifier;
      this.negativeModifier = negativeModifier;
      this.attackSpeed = attackSpeed;
    }

    public String getWeaponNameSuffix() {
      return weaponNameSuffix;
    }

    public float getPositiveModifier() {
      return positiveModifier;
    }

    public float getNegativeModifier() {
      return negativeModifier;
    }

    public float getAttackSpeed() {
      return attackSpeed;
    }
  }

  public static final DeferredRegister<Item> ITEMS = DeferredRegister.create(
    SimplyCompat.MOD_ID,
    Registries.ITEM
  );

  private static RegistrySupplier<SimplySwordsSwordItem> registerSword(
    String materialPrefix,
    SimplyCompatToolMaterials material,
    WeaponType type
  ) {
    // Get the base modifier from config by constructing the key "<materialPrefix>_damageModifier"
    // float baseModifier = Config.getFloat(
    //   materialPrefix + "_damageModifier",
    //   "WeaponAttributes",
    //   1.0F // TODO: Change config to a data structure, or at least have one available for accessing here, Since we cant `ConfigDefaultValues.material_damageModifier` here
    // );

    var materialTag = material.getIdentifier();

    float finalDamage = 2.0f + type.getPositiveModifier() - type.getNegativeModifier();
    // float finalDamage = baseModifier + type.getPositiveModifier() - type.getNegativeModifier();
    String name = materialPrefix + "_" + type.getWeaponNameSuffix();

    return ITEMS.register(name, () ->
      new SimplySwordsSwordItem(material, (int) finalDamage, type.getAttackSpeed(), materialTag)
    );
  }

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_CHAKRAM = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.CHAKRAM
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_CLAYMORE = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.CLAYMORE
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_CUTLASS = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.CUTLASS
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_GLAIVE = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.GLAIVE
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_GREATAXE = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.GREATAXE
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_GREATHAMMER = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.GREATHAMMER
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_HALBERD = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.HALBERD
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_KATANA = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.KATANA
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_LONGSWORD = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.LONGSWORD
  );
  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_RAPIER = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.RAPIER
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_SAI = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.SAI
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_SCYTHE = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.SCYTHE
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_SPEAR = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.SPEAR
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_TWINBLADE = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.TWINBLADE
  );

  public static final RegistrySupplier<SimplySwordsSwordItem> COPPER_WARGLAIVE = registerSword(
    "copper",
    SimplyCompatToolMaterials.COPPER,
    WeaponType.WARGLAIVE
  );
}
